@@ -1,8 +1,11 @@
 # SPDX-License-Identifier: BSD-2-Clause
 
-# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>
+# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>
 
 
+from ...exceptions import UserException
+from ...module import parse_abi_version
+
 from ..scoped_name import ScopedName
 from ..specification import (AccessSpecifier, Argument, ArgumentType,
         ArrayArgument, ClassKey, Docstring, DocstringFormat, Extract,
@@ -72,6 +75,7 @@ def p_statement(p):
         | license
         | mapped_type
         | mapped_type_template
+        | minimum_abi_version
         | module
         | module_code
         | module_header_code
@@ -89,6 +93,7 @@ def p_statement(p):
 def p_namespace_statement(p):
     """namespace_statement : if_start
         | if_end
+        | namespace_docstring
         | class_decl
         | class_template
         | enum_decl
@@ -865,6 +870,7 @@ def p_license_arg(p):
 # The mapped type annotations.
 _MAPPED_TYPE_ANNOTATIONS = (
     'AllowNone',
+    'Movable',
     'NoAssignmentOperator',
     'NoCopyCtor',
     'NoDefaultCtor',
@@ -991,6 +997,41 @@ def p_mapped_type_function(p):
     pm.validate_function(p, 1, overload)
 
 
+# %MinimumABIVersion ##########################################################
+
+def p_minimum_abi_version(p):
+    """minimum_abi_version : MinimumABIVersion STRING
+        | MinimumABIVersion begin_args '(' version '=' STRING end_args ')'"""
+
+    pm = p.parser.pm
+
+    if pm.skipping:
+        return
+
+    symbol = 2 if len(p) == 3 else 6
+
+    try:
+        min_major_version, min_minor_version = parse_abi_version(p[symbol])
+    except UserException as e:
+        pm.parser_error(p, symbol, e.text)
+        return
+
+    if pm.spec.target_abi is None:
+        pm.spec.target_abi = (min_major_version, min_minor_version)
+    else:
+        major_version, minor_version = pm.spec.target_abi
+
+        if major_version != min_major_version:
+            pm.parser_error(p, symbol,
+                    f"v{min_major_version} is specified but v{major_version} is being targeted")
+        elif min_minor_version is not None:
+            if minor_version is None:
+                pm.spec.target_abi = (min_major_version, min_minor_version)
+            elif min_minor_version is not None and minor_version < min_minor_version:
+                pm.parser_error(p, symbol,
+                        f"v{min_major_version}.{min_minor_version} is the minimum ABI version but v{major_version}.{minor_version} is being targeted")
+
+
 # %ModuleHeaderCode ###########################################################
 
 def p_module_header_code(p):
@@ -1833,7 +1874,7 @@ def p_superclass_list(p):
 
 
 def p_superclass(p):
-    "superclass : class_access scoped_name"
+    "superclass : class_access simple_superclass"
 
     pm = p.parser.pm
 
@@ -1844,21 +1885,7 @@ def p_superclass(p):
         p[0] = None
         return
 
-    # This is a hack to allow typedef'ed classes to be used before we have
-    # resolved the typedef definitions.  Unlike elsewhere, we require that the
-    # typedef is defined before being used.
-    ad = Argument(ArgumentType.DEFINED, definition=p[2])
-
-    while ad.type is ArgumentType.DEFINED:
-        ad.type = ArgumentType.NONE
-        search_typedefs(pm.spec, ad.definition, ad)
-
-    if ad.type is not ArgumentType.NONE or len(ad.derefs) != 0 or ad.is_const or ad.is_reference:
-        pm.parser_error(p, 2, "super-class list contains an invalid type")
-
-    # Find the actual class.
-    p[0] = pm.find_class(p, 2, IfaceFileType.CLASS, ad.definition,
-            tmpl_arg=pm.parsing_template)
+    p[0] = p[2]
 
 
 def p_class_access(p):
@@ -1873,6 +1900,43 @@ def p_class_access(p):
     p[0] = p[1]
 
 
+def p_simple_superclass(p):
+    """simple_superclass : scoped_name
+        | scoped_name '<' cpp_types '>'"""
+
+    pm = p.parser.pm
+
+    # Handle templates.
+    if len(p) == 5:
+        if pm.parsing_template:
+            # For the moment we just remember enough to identify it later when
+            # we have the values to instantiate it.
+            p[0] = Argument(ArgumentType.TEMPLATE,
+                    definition=Template(p[1], Signature(args=p[3])),
+                    source_location=pm.get_source_location(p, 1))
+        else:
+            pm.parser_error(p, 1,
+                    "super-class templates can only be used in a class template")
+
+        return
+
+    # This is a hack to allow typedef'ed classes to be used before we have
+    # resolved the typedef definitions.  Unlike elsewhere, we require that the
+    # typedef is defined before being used.
+    ad = Argument(ArgumentType.DEFINED, definition=p[1])
+
+    while ad.type is ArgumentType.DEFINED:
+        ad.type = ArgumentType.NONE
+        search_typedefs(pm.spec, ad.definition, ad)
+
+    if ad.type is not ArgumentType.NONE or len(ad.derefs) != 0 or ad.is_const or ad.is_reference:
+        pm.parser_error(p, 1, "super-class list contains an invalid type")
+
+    # Find the actual class.
+    p[0] = pm.find_class(p, 1, IfaceFileType.CLASS, ad.definition,
+            tmpl_arg=pm.parsing_template)
+
+
 def p_opt_class_definition(p):
     """opt_class_definition : '{' opt_class_body '}'
         | empty"""
@@ -2147,28 +2211,52 @@ _ENUM_MEMBER_ANNOTATIONS = (
 
 
 def p_enum_decl(p):
-    "enum_decl : enum opt_enum_key opt_name opt_annos '{' opt_enum_body '}' ';'"
+    "enum_decl : enum opt_enum_key opt_name opt_enum_base opt_annos '{' opt_enum_body '}' ';'"
 
     pm = p.parser.pm
 
     if pm.skipping:
         return
 
-    pm.check_annotations(p, 4, "enum", _ENUM_ANNOTATIONS)
+    pm.check_annotations(p, 5, "enum", _ENUM_ANNOTATIONS)
 
-    pm.add_enum(p, 1, p[3], p[2], p[4], p[6])
+    pm.add_enum(p, 1, p[3], p[2], p[4], p[5], p[7])
 
 
 def p_opt_enum_key(p):
     """opt_enum_key : class
         | struct
-        | union
         | empty"""
 
+    pm = p.parser.pm
+
+    if pm.skipping:
+        return
+
+    if p[1] is not None:
+        pm.cpp_only(p, 1, "enum keys")
+
     # Return True if the enum is scoped.
     p[0] = p[1] is not None
 
 
+def p_opt_enum_base(p):
+    """opt_enum_base : ':' base_type
+        | empty"""
+
+    pm = p.parser.pm
+
+    if pm.skipping:
+        return
+
+    if len(p) == 3:
+        pm.cpp_only(p, 1, "enum bases")
+
+        p[0] = p[2]
+    else:
+        p[0] = None
+
+
 def p_opt_enum_body(p):
     """opt_enum_body : enum_body
         | empty"""
@@ -2975,6 +3063,20 @@ def p_namespace_body(p):
     """namespace_body : namespace_statement
         | namespace_body namespace_statement"""
 
+def p_namespace_docstring(p):
+    "namespace_docstring : docstring"
+
+    pm = p.parser.pm
+
+    if pm.skipping:
+        return
+
+    if pm.scope.docstring is None:
+        pm.scope.docstring = p[1]
+    else:
+        pm.parser_error(p, 1,
+                "%Docstring has already been defined for this namespace")
+
 
 # C/C++ typedefs. #############################################################
 
